# -*- coding: utf-8 -*-
import re
from copy import copy

from django.contrib import admin
from django.contrib.admin.views.main import ChangeList

import django_filters

class AdvancedSearchModelAdmin(admin.ModelAdmin):

    def get_filterset_class(self, request):
        """Return a filterset_class suitable for advanced search
        first look if attribute ``advanced_search_fields`` exists or
        :meth:`get_filterset_class` exists
        """
        fields = self.get_advanced_search_fields(request) 
        
        if fields:
            meta = type('Meta', (object,), {'fields': fields, 'model' : self.model})
            AdminFilterset = type('Admin%sFilterset' % self.model.__name__, 
                                  (django_filters.FilterSet,),
                                  { 'Meta' : meta, 'fields' : fields})
            return AdminFilterset
        # elif hasattr(self, 'get_filterset_class'):
        #     return self.get_filterset()
        return None


    def get_advanced_search_fields(self, request):
        """
        Return the list of fields that should be used in advanced search

        :arg request: the request
        """
        return getattr(self, 'advanced_search_fields', None)

    def lookup_allowed(self, lookup, value):
        if hasattr(self, '_lookup_search'):
            if lookup in self._lookup_names:
                return True
        return super(AdvancedSearchModelAdmin, self).lookup_allowed(lookup, value)

    def get_changelist(self, request):
        return AdvancedSearchChangeList

    def queryset(self, request):
        search_filterset_class = self.get_filterset_class(request)

        if search_filterset_class:
            self.search_filterset = search_filterset_class(request.GET)
            self.search_form = self.search_filterset.form
            self.search_form.is_valid()
            self._lookup_names = re.findall('name="(?P<name>[^\"]*)"', str(self.search_form))

            return self.search_filterset.qs
        return admin.ModelAdmin.queryset(self, request)
            
class AdvancedSearchChangeList(ChangeList):

    def get_query_set(self, request):

        # save a copy of the GET parameters 
        saved_params = copy(self.params)

        if hasattr(self.model_admin, 'search_form'):
            search_form = self.model_admin.search_form
            search_form.is_valid()

            if search_form.is_valid():
                for name in self.model_admin._lookup_names:
                    self.params.pop(name, None)

        # Hide all parameters taht are handled by the filterset
        # directly that's the only part that knows how to handle it
        qs_result = ChangeList.get_query_set(self, request)

        # restore the self.params list so that templatetag 'result_list'
        # can compose it correctly. In particular ordering and filters
        # need to know all parameters
        self.params = saved_params

        return qs_result
